/*	date.cpp
Ken Bloom
Version 1.0 lesson 30
*/
#include "config.h"
#include <string>
#include <cstdlib> // for atoi
#include <iostream>
#include <cassert>
#include <ctime>
#include "date.h"
namespace kenDate{
	using namespace std;
	////////////////////////////////////////////////////////////
	// Special prototypes private to this file
	inline bool isDivisible(long divisor, long dividend);
	inline int numDigits(int x);
	
	////////////////////////////////////////////////////////////
	// Constructors
	date::date()
	{
		time_t t;
		time(&t);
		tm* today=localtime(&t);
		short m,d,y;
		m=today->tm_mon+1;
		d=today->tm_mday;
		y=today->tm_year+1900;
		setDate(m,d,y);
	}
	date::date(short month,short day,short year)
	{
		setDate(month,day,year);
	}
	
	///////////////////////////////////////////////////////////
	// Modifiers
	void date::increaseDays()
	{
		if (myDay==daysInMonth(myMonth,myYear))
		{
			if (myMonth==12)
			{
				myMonth=1;
				myYear++;
				myDay=1;
			}
			else
			{
				myMonth++;
				myDay=1;
			}
		}
		else
		{
			myDay++;
		}
	}
	
	void date::setDate (short month, short day, short year)
	{
	/* note that in this function we don't actually change any dates until we're sure they're
		all valid. */
		if (!(0<month&&month<=12))
			throw badDate();
		
		if (!(0<day&&day<=daysInMonth(month,year)))
			throw badDate();
		
		if (year>=100) {}
		else if (year>=20) year+=1900;
		else if (year>=0) year+=2000;
		else throw badDate();
		
		/* once we're here, either the date is valid or we've thrown an exception to get out. */
		myMonth=month;
		myDay=day;
		myYear=year;
	}
	
	void date::getDate (istream& in) // default parameter
	{
		string theDate;
		short m,d,y;
		in >> theDate;
		if (!in) throw badDate();
		
		if (isInSlashSlashFormat(theDate))
		{
			splitString(theDate,m,d,y);
		}
		else
		{
			// now we assume that user entered 1 1 2000 format
			// the input operator stops at the first space it encounters so:
			
			m=atoi(theDate.c_str()); // got the month in the string, convert it to int
			if (m==0) throw badDate(); // this line is analagous to the error parsing below
			
			cin >> d; // we still have to get the day
			if (!in) throw badDate();
			
			cin >> y; // and get the year
			if (!in) throw badDate();
		}
		
		setDate(m,d,y); // may throw if date is bad data
	}
	///////////////////////////////////////////////////////////
	// Arithmetic Operators
	date& date::operator++()
	{
		increaseDays();
		return *this;
	}

	date& date::operator--()
	{
		if (myDay==1)
		{
			if (myMonth==1)
			{
				myYear--;
				myMonth=12;
			}
			else
			{
				myMonth--;
			}
			myDay=daysInMonth (myMonth,myYear);
		}
		else
		{
			myDay--;
		}
		return *this;
	}
	
	date& date::operator+=(signed long d)
	{
		if (d<0) return (*this)-=(-d);
		if (d>0)
		{
			for (signed long t=0;t<d;t++)
			{
				(*this)++;
			}
		}
		return *this;
	}
	date& date::operator-=(signed long d)
	{
		if (d<0) return (*this)+=(-d);
		if (d>0)
		{
			for (signed long t=0;t<d;t++)
			{
				(*this)--;
			}
		}
		return *this;
	}
	
	///////////////////////////////////////////////////////////
	// Accessors
	void date::print(ostream& out) const // has default parameter
	{
		int tlen=numDigits(myMonth)+
			     numDigits(myDay)+
			     numDigits(myYear)+2;
		
		//Save iostream formatting settings
		const ostream::fmtflags savedflags=out.flags();
		const streamsize savedwidth=out.width();

		// kludge to handle conflicts between ios::left and ios::right
		ostream::fmtflags operative=savedflags;
		if (operative & ios::left) operative = operative & ~ios::right;

		//Set iostream formatting to something I can use
		out.flags(ios::left);
		out.width(0);
		
		//pad if right aligned
		if(operative & ios::right)
		{
			for (int ctr=0;ctr<savedwidth-tlen;ctr++)
				out.put(out.fill());
		}

		//output the date
		out << myMonth << '/' << myDay << '/' << myYear;

		//pad if left aligned
		if(operative & ios::left)
		{
			for (int ctr=0;ctr<savedwidth-tlen;ctr++)
				out.put(out.fill());
		}

		//restore changed iostream formatting
		out.flags(savedflags);
	}
	
	short date::month() const
	{
		return myMonth;
	}
	
	short date::day() const
	{
		return myDay;
	}
	
	short date::year() const
	{
		return myYear;
	}

	///////////////////////////////////////////////////////////
	// Time Fields. (ignored in calculations)
	short date::hour() const
	{
	 return myHour;
	}
	void date::hour(short theHour){
	 myHour=theHour;
	 }
	short date::min() const
	{
	 return myMin;
	}
	void date::min(short theMin){
	 myMin=theMin;
	 }

	///////////////////////////////////////////////////////////
	// I/O Operators
	ostream& operator<<(ostream& out, const date& d)
	{
		d.print(out);
		return out;
	}
	
	istream& operator>>(istream& in, date& d)
	{
		if (!in) return in; // if failbit is set, don't try to read anything from the stream 																				
		try
		{
			d.getDate(in);
		}
		catch (...)
		{
			in.setstate(ios::failbit);
			/* this is the most ANSI standard procedure for handling errors
			reading from a stream. IF THIS OPERATOR SETS failbit,
			THE CALLING CODE MUST EXECUTE THE clear() FUNCTION OF THE STREAM
			IN ORDER TO CONTINUE TO USE IT.*/
		}
		return in;
	}
	
	///////////////////////////////////////////////////////////
	// Relational Operators
	bool date::operator < (const date& right) const
	{
		const date& left=*this;
	
		if (left.year()<right.year())
			return true;
		else if (left.year()==right.year())
		{
			if (left.month()<right.month())
				return true;
			else if (left.month()==right.month())
			{
				if (left.day()<right.day())
					return true;
				else return false;
			}
			else return false;
		}
		else return false;
	}
	
	bool date::operator == (const date& right) const
	{
		const date& left=*this;

		return (
			(left.day()==right.day())
			&&(left.month()==right.month())
			&&(left.year()==right.year())
			);
	}
	
	///////////////////////////////////////////////////////////
	// Arithmetic Operators

	signed long operator-(date left, date right)
	{
#ifndef NDEBUG
		date oldleft=left,oldright=right;
		// only need to do this when the assertion is evaluated
#endif
		signed long x=0;

		while (left<right) // automatically does the if case
		{
			left++;
			x--;
		}
		
		while (left>right) // automatically does the if case
		{
			right++;
			x++;
		}

		assert (oldleft==oldright+x); // check for the identity
		return x;
	}
	
	
	///////////////////////////////////////////////////////////
	// Utility
	short date::daysInMonth (short month, short year) const // private
	{
		assert(0<month&&month<=12);
		switch (month)
		{
		case 1: case 3: case 5: case 7:
		case 8: case 10:  case 12:
			return 31;
		case 4: case 6: case 9: case 11:
			return 30;
		case 2: // February is a special and very confusing case.
			if (isDivisible (year,4))
				if (isDivisible(year,100))
					if (isDivisible(year,400))
						return 29;
					else
						return 28;
					else
						return 29;
					else
						return 28;
		}
		return 0; // never used, but the compiler won't know
	}
	
	bool date::isInSlashSlashFormat (string attempt) const // private
	{
		int loc=attempt.find('/');
		if (loc!=string::npos)
		{
			attempt[loc]='*';
			if (attempt.find('/')!=string::npos) return true;
		}
		return false;
	}
	
	void date::splitString (string theDate, short& m, short& d, short& y) const //private
	{
		string tempBuffer;
		int currentSpot;
		int readTo;
		
		////// Month
		readTo=theDate.find('/'); // find delimiter
		for (currentSpot=0;currentSpot<readTo;currentSpot++) // fill buffer - this is easier than
			tempBuffer+=theDate[currentSpot];                // using string::substr()
		m=atoi(tempBuffer.c_str()); // translate buffer
		theDate[readTo]='*'; // cleanup
		tempBuffer="";
		
		////// Day
		readTo=theDate.find('/'); //find delimiter
		for (currentSpot++;currentSpot<readTo;currentSpot++) // fill buffer
			tempBuffer+=theDate[currentSpot];
		d=atoi(tempBuffer.c_str()); // translate buffer
		tempBuffer=""; //cleanup
		
		////// Year
		for (currentSpot++;currentSpot<theDate.length();currentSpot++) // fill buffer
			tempBuffer+=theDate[currentSpot];
		y=atoi(tempBuffer.c_str()); // translate buffer
		
	}
	
	inline bool isDivisible(long divisor, long dividend)
	{
		if ((divisor%dividend)==0)
			return true;
		else
			return false;
	}

	inline int numDigits(int x)
	{
		int digits=0;
		do{
			x/=10;
			digits++;
		}while(x>0);
		return digits;
	}
}

