/*    This file is part of Zmanim.

    Zmanim is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Zmanim is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Zmanim; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/*	date.h
Ken Bloom
Version 1.0 lesson 30
*/
#ifndef __k_DATE_H__
#define __k_DATE_H__

#include "config.h"
#include <iostream>
#include <string>
#include <exception>
#include <stdexcept>
#include "operators.h"

namespace kenDate{
	////////////////////////////////////////////////////////////////////////////////
	//////////////////// Exceptions ////////////////////////////////////////////////
	struct badDate:public std::runtime_error
	{
		badDate():std::runtime_error(""){}	
	};
	
	////////////////////////////////////////////////////////////////////////////////
	//////////////////// Class Declaration /////////////////////////////////////////
	class date
		:public kenOps::comparable1    < date >
		,public kenOps::dist_operators < date, signed long >
		,public kenOps::post_inc_dec   < date >
	{
	public:
		////////////////////////////////////////////////////////////////////////////////
		//////////////////// Constructors //////////////////////////////////////////////
		date();
		/*
		Post: Constructs a date object representing the current date
		*/
		date(short month,short day,short year);
		/*
		Pre:0<month<=12,day is in the range of days for the given month and year
		See notes for setDate(short,short,short) for cases where 0<=year<=99
		Post:Date is initialized to the given values. If invalid, a badDate
		exception is thrown.
		*/
		////////////////////////////////////////////////////////////////////////////////
		//////////////////// Modifiers /////////////////////////////////////////////////
		void setDate (short month, short day, short year);
		/*
		Pre:0<month<=12,day is in the range of days for the given month and year
		The date class may handle a special case when 0<=year<=99, but
		is not required to. The current implimentation of the class handles
		these these years in an undefined special manner, but in a future
		version may map these years to the years 0..99
		Post:Date is set to the given values. If the new date is invalid,
		a badDate exception is thrown
		*/
		void increaseDays();
		/*
		Deprecated. Same as either operator++ but does not return any value.
		*/
		////////////////////////////////////////////////////////////////////////////////
		//////////////////// Arithmetic Operators //////////////////////////////////////
		
		date& operator++();
		/*
		Act:Increments the date.
		Post: returns the new value of the date
		*/
		date& operator--();
		/*
		Act:Decrements the date.
		Post: returns the new value of the date
		*/
		date& operator+=(signed long d);
		/*
		Act: Increases the date to the date d days after the date of the current object
		Post: returns the new date.
		*/
		date& operator-=(signed long d);
		/*
		Act: Decreases the date to the date d days before the date of the current object
		Post: returns the new date.
		*/
		////////////////////////////////////////////////////////////////////////////////
		//////////////////// Relational Operators //////////////////////////////////////
		bool operator < (const date& right) const;
		/*
		Post: Returns true if *this occurs chronologically before right.
		Returns false, otherwise.
		*/
		bool operator == (const date& right) const;
		/*
		Post: Returns true if *this is the same date as right.
		Returns false, otherwise.
		*/
		
		////////////////////////////////////////////////////////////////////////////////
		//////////////////// Accessors /////////////////////////////////////////////////
		short month() const;
		short day() const;
		short year() const;
		/*
		These three accessors return the month, day and year respectively.
		*/

		////////////////////////////////////////////////////////////////////////////////
		//////////////////// Time functions (ignored in calculations)///////////////////
		short hour() const;
		void hour(short theHour);
		short min() const;
		void min(short theMin);

	private:
		friend std::ostream& operator<<(std::ostream& out, const date& d);
		friend std::istream& operator>>(std::istream& in, date& d);

		short myMonth, myDay,myYear,myHour,myMin;

		void getDate (std::istream& in = std::cin);
		/*
		Act: Sets the date by reading from in. This function is
		defined to always accept the date in the format 1/1/2000 or 1/1/00
		or 1 1 2000 or 1 1 00, but may accept the date in more formats
		in later versions.
		Post: Throws badDate if the date input is invalid. In the event of such an error,
		whether this function attempts to put the bad data back on in is undefined.
		This function is completely compatible with and replaces the function getDate(void)
		from previous versions. No calling modifications are necessary.
		*/
		void print(std::ostream& out=std::cout) const;
		/*
		Act: Writes the date to the stream out. The format in which the date is
		written is undefined. The print function will never, under any circumstances,
		output the date in a format that that version's getDate function can not read
		back from the stream.
		*/
		short daysInMonth (short month, short year) const;
		// This function performs no modificaton on the year before checking the year.
		bool isInSlashSlashFormat (std::string attempt) const;
		// Tells me if my string is a single integer (the month) or the whole date.
		void splitString (std::string theDate, short& m, short& d, short& y) const;
		// performs conversions on a string known to be in 1/1/2000 or 1/1/00 format
	};
	////////////////////////////////////////////////////////////////////////////////
	//////////////////// I/O Operators /////////////////////////////////////////////
	std::ostream& operator<<(std::ostream& out, const date& d);
	/*
	Pre: The stream out is opened and in a valid state.
	Act: Writes the date to the stream out. The format in which the date is
	written is undefined. The operator<< will never, under any circumstances,
	output the date in a format that that version's operator>> can not read
	back from the stream.
	Post: Returns out. This function is defined to have no error conditions.
	*/
	
	std::istream& operator>>(std::istream& in, date& d);
	/*
	Pre: If `in.fail()', the function does not attempt any read operation.
	If `!in.fail()', operation will proceed normally.
	Act: Sets the date by reading from `in'. This operator is
	defined to always accept the date in the format 1/1/2000 or 1/1/00
	or 1 1 2000 or 1 1 00, but may accept the date in more formats
	in later versions.
	Post: In the event of an error, sets ios::failbit in the state. THE CALLER
	WILL HAVE TO CALL THE STREAM'S clear() FUNCTION FOR THE INPUT OPERATORS
	OF THE STREAM TO OPERATE AFTER AN ERROR OCCURS. 
	To check for an error condition:
	if (i>>d)
	{
	// do code if it succeds
	}
	else
	{
	// do code if it fails
	}
	
	  If no error occurs, the function returns `in'.
	*/
	
	////////////////////////////////////////////////////////////////////////////////
	//////////////////// Free Arithmetic Operators /////////////////////////////////
	signed long operator-(date left, date right);
	/*
	Post: Returns the number of days that the date "right" occurs before the date "left".
	The distance is defined as the difference between any given time on the date "right"
	and the same time on the date "left." For example, date(1,1,96)-date(12,31,95) == 1
	and date(12,31,95)-date(1,1,96) == -1 .
	
	  This operator and the + operator conform to the identity x==d+(x-d)
	*/
}

#endif
