/*    This file is part of Zmanim.

    Zmanim is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Zmanim is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Zmanim; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/*	Day.h
Ken Bloom
*/
#include "config.h"
#include <string>
#include <iostream>
#include "date.h"

#ifndef __k_DAY_H__
#define __k_DAY_H__
namespace kenDate{
	enum day {sunday=1,monday,tuesday,wednesday,thursday,friday,saturday};
	
	std::string dayName(day d);
	/*
	Translates a day into a string
	*/
	
	day& operator++(day& d);
	/*
	Increments the day to the next day of the week.
	Returns the new day.
	*/
	
	day operator++(day& d,int);
	/*
	Increments the day to the next day of the week.
	Returns the old day.
	*/
	
	day& operator--(day& d);
	/*
	Decrements the day to the previous day of the week.
	Returns the new day.
	*/
	
	day operator--(day& d,int);
	/*
	Decrements the day to the previous day of the week.
	Returns the old day.
	*/
	
	day& operator+=(day& d, signed long numDays);
	/*
	Sets d to the day numDays later than d and returns the new value.
	*/
	
	day& operator-=(day& d, signed long numDays);
	/*
	Sets d to the day numDays earlier than d and returns the new value.
	*/
	
	day operator+(day d, signed long numDays);
	/*
	Returns the day numDays earlier than d.
	*/
	
	day operator-(day d, signed long numDays);
	/*
	Returns the day numDays earlier than d.
	*/
	
	/*
	Note that there is no day-day operator. Because the desired behaviour of this
	operator is ambiguous, it is defined to not be included. The user nust roll
	their own.
	*/
	
	std::ostream& operator<<(std::ostream& out, const day& theDay);
	/*
	Pre:The ostream is in a valid state.
	Act:Writes the day name to the ostream
	Post:Returns a reference to the ostream
	*/
	
	std::istream& operator>>(std::istream& in, day& theDay);
	/*
	Pre:The istream is in a valid state.
	Act:Reads the day name from the istream
	Post:Returns a reference to the istream
	*/
	
	day dayOf(date d);
	/*
	Act:Returns the day of the week on which d occurs.
	*/
}
#endif
