// vim: fdc=5 fdm=syntax
/*    This file is part of Zmanim.
 
  Zmanim is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  Zmanim is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with Zmanim; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "config.h"
#include "sunfuncs.h"
#include "date.h"
#include "day.h"
#include <iostream>
#include <fstream>
#include <cmath>
#include <cstdio>
#include <string>
#include <cstdlib>
#include <cassert>

using namespace kenDate;

using namespace std;

struct config_file_error {
   enum state {PARSE_ERROR, ENTRY_NOT_FOUND, FILE_NOT_FOUND};
   state status;
   config_file_error( state s ) : status( s ) {}}

;

struct location_info {
   double lat; //latitude
   double longi; //longitude
   string tzname;
   string name;
};

struct times_t {
   double alot_hashachar_dbl, mesheyakir_dbl, hanetz_dbl, shkiah_dbl, tzait_hachochavim_dbl;
   date alot_hashachar, mesheyakir, hanetz, shkiah, tzait_hachochavim;
   date zmanshema, zmantefila, chatzot, minchagedola, minchakatanah, plaghamincha, candles18;
   double ph;
};

void usagemessage( string progname );
void parseopts ( int argc, char** argv, location_info& allinfo, date& thedate );
void parseconfigfile( string& locationname, location_info& allinfo );
times_t computetimes( location_info const & locinfo, date thedate );
inline void printtime( date time );
bool isDST( date thedate );

//prototypes for output functions
void oneday_longform ( location_info const & locinfo, date thedate);

//use this to print time:
//printf( "%.2d:%.2d\n" ,hour, min)
//printf( "%.2d:%.2d\n" ,int(floor(sset)),int((sset-floor(sset))*60)) //where sset is variable

int main( int argc, char** argv ) {
   date thedate;
   location_info locinfo;

   try {
      parseopts ( argc, argv, locinfo, thedate );

      { //header
         cout << "======================================================" << endl;
         cout << "======================================================" << endl;
         cout << "Zmanim for " << locinfo.name << endl;
         cout << endl;
      }
      oneday_longform(locinfo, thedate);
      return 0;

   } 
   catch ( doesnthappen e){
      cerr << "Either sunrise or sunset (or both) doesn't happen at " << endl
           << "this latitude on this day." << endl;
   }
   catch ( ... ) {
      usagemessage( argv[ 0 ] );
      return 1;
   }
}

void oneday_longform ( location_info const & locinfo, date thedate){


      { //yesterday
	 date d=thedate-1;
	 times_t t=computetimes(locinfo, d );

         cout << "======================================================" << endl;
         cout << "=== " << dayOf ( d ) << ' ' << d << endl;
         cout << "GRA     Proportional Hour                       " << int( t.ph * 60 ) << " minutes" << endl;
         cout << "                             Candle Lighting    "; printtime( t.candles18); cout << endl;
         cout << "        Shkiah               Sunset         *** "; printtime( t.shkiah ); cout << endl;
         cout << "8.5deg  Tzait HaChochavim    Nightfall          "; printtime( t.tzait_hachochavim ); cout << endl;
         cout << endl;
      }

      { //today
	 times_t t=computetimes(locinfo, thedate);

         cout << "======================================================" << endl;
         cout << "=== " << dayOf ( thedate ) << ' ' << thedate << endl;
         cout << "GRA     Proportional Hour                       " << int( t.ph * 60 ) << " minutes" << endl;
         cout << "16.1deg Alot HaShachar       Daybreak           "; printtime( t.alot_hashachar ); cout << endl;
         cout << "11 deg  Mesheyakir           Earliest Talit     "; printtime( t.mesheyakir ); cout << endl;
         cout << "        Hanetz HaChama       Sunrise        *** "; printtime( t.hanetz ); cout << endl;
         cout << "GRA     Sof Zman Sh'ma       Latest Sh'ma       "; printtime( t.zmanshema); cout << endl;
         cout << "GRA     Sof Zman T'fila      Latest T'fila      "; printtime( t.zmantefila); cout << endl;
         cout << "        Chatzot              Midday             "; printtime( t.chatzot); cout << endl;
         cout << "GRA     Mincha Gedola                           "; printtime( t.minchagedola); cout << endl;
         cout << "GRA     Mincha Katanah                          "; printtime( t.minchakatanah) ; cout << endl;
         cout << "GRA     Plag Hamincha                           "; printtime( t.plaghamincha); cout << endl;
         cout << "                             Candle Lighting    "; printtime( t.candles18); cout << endl;
         cout << "        Shkiah               Sunset         *** "; printtime( t.shkiah ); cout << endl;
         cout << "8.5deg  Tzait HaChochavim    Nightfall          "; printtime( t.tzait_hachochavim ); cout << endl;
      }

}

void parseopts ( int argc, char** argv, location_info& allinfo, date& thedate ) {
   //find out what format we used to choose location and parse it
   string thedate_unparsed;

   switch ( argc ) {

   case 1:    //progname

   case 2:    // progname, date
      allinfo.name = string( "default" );
      parseconfigfile( allinfo.name, allinfo );
      //allinfo.name is changeable so the name of the default
      //will be printed, and not "default"
      break;

   case 3:
      allinfo.name = argv[ 2 ];
      parseconfigfile( allinfo.name, allinfo );
      break;

   case 5:    //progname, date, lat, longi, tzname
      allinfo.lat = atof( argv[ 2 ] );
      allinfo.longi = atof( argv[ 3 ] );
      allinfo.tzname = argv[ 4 ] ;
      break;

   default:
      throw 1;
   }

   if ( argc == 1 ) {
      thedate_unparsed = string( "today" );
   } else {
      thedate_unparsed = argv[ 1 ];
   }


   if ( thedate_unparsed == "shabbat" ) {
      while ( dayOf( thedate ) != saturday )
         thedate++;
   } else if ( thedate_unparsed == "today" )

      ; //thedate is already correct
   else if ( thedate_unparsed == "tomorrow" ) {
      thedate++;
   } else if ( thedate_unparsed.find( '/' ) != string::npos ) {
      int month, day, year;
      char buf[ 20 ];
      strncpy( buf, thedate_unparsed.c_str(),20 );
      buf[19]='\0';
      month = atoi( strtok( buf, "/" ) );
      day = atoi( strtok( 0, "/" ) );
      year = atoi( strtok( 0, "/" ) );
      thedate = date( month, day, year );
   } else
      throw 1; //the parameter is invalid
}

void parseconfigfile( string& locationname, location_info & locinfo ) {
   locinfo.name = locationname;
   double& latitude = locinfo.lat;
   double& longitude = locinfo.longi;
   string& tzname = locinfo.tzname;
   string conffile = string( getenv( "HOME" ) ) + "/.zmanrc";
   ifstream infile( conffile.c_str() );
   infile.exceptions( ios::badbit | ios::failbit | ios::eofbit );
   string cur_row_name;

   while ( infile >> cur_row_name ) {
      if ( cur_row_name == "default" ) {
         if ( locationname == "default" ) {
            infile >> locationname;
            break;
         } else {
            infile >> cur_row_name;
         }
      }

      if ( cur_row_name == locationname ) {
         break;
      }

      infile.ignore( 1024, '\n' );
   }

   if ( !infile )
      throw config_file_error( config_file_error::ENTRY_NOT_FOUND ); //made it to end of file

   infile >> latitude >> longitude >> tzname;
}

void usagemessage( string progname ) {
   cerr << "usage: use one of the following forms." << endl;
   cerr << "   " << progname << " #uses default location from $HOME/.zmanrc and 'today'" << endl;
   cerr << "   " << progname << " <date> #uses default location from $HOME/.zmanrc" << endl;
   cerr << "   " << progname << " <date> <location_name> #from $HOME/.zmanrc" << endl;
   cerr << "   " << progname << " <date> <lat> <long> <tzname>" << endl;
   cerr << endl;
   cerr << "format for $HOME/.zmanrc" << endl;
   cerr << "[default] <location_name> <lat> <long> <tzname>" << endl;
   cerr << endl;
   cerr << "use of 'dst' means to observe daylight savings time at the appropriate dates" << endl;
   cerr << "latitude is negative if south. longitude is negative if west." << endl;
   cerr << endl;
   cerr << "enter date in mm/dd/yyyy format, enter 'shabbat' for the upcoming shabbat, or enter 'today' or 'tomorrow'." << endl;
}

inline void printtime( date time ) {

   if (time.hour() == -1 && time.min() == -1 ){
      printf("N/A");
      return;
   }

   char const * ampm = "am";

   if ( time.hour() == 12 && time.min()==0 )
      ampm = "noon";

   if ( time.hour() == 0 ){
      ampm = "midnight";
      time.hour(12);
   }

   if ( time.hour() > 12 ){
      ampm = "pm";
      time.hour(time.hour()-12);
   }

   if (time.hour()==12 && time.min()!=0){
      ampm="pm";
   }

   printf( "%d:%.2d %s" , time.hour() , time.min() , ampm );
}

times_t computetimes( location_info const & locinfo, date thedate ) {
   times_t result;

   try{
      result.alot_hashachar_dbl = sunrise ( thedate, locinfo.lat, locinfo.longi, 90 + 16.1 );
      result.alot_hashachar = convert_to_tz(thedate,result.alot_hashachar_dbl,locinfo.tzname);
   }catch(doesnthappen e){
      result.alot_hashachar = thedate;
      result.alot_hashachar.hour(-1);
      result.alot_hashachar.min(-1);
   }
   try{
      result.mesheyakir_dbl = sunrise ( thedate, locinfo.lat, locinfo.longi, 90 + 11 );
      result.mesheyakir = convert_to_tz(thedate,result.mesheyakir_dbl,locinfo.tzname);
   }catch(doesnthappen e){
      result.mesheyakir = thedate;
      result.mesheyakir.hour(-1);
      result.mesheyakir.min(-1);
   }
   result.hanetz_dbl = sunrise( thedate, locinfo.lat, locinfo.longi, 90 + .83 );
   result.hanetz = convert_to_tz(thedate,result.hanetz_dbl,locinfo.tzname);
   result.shkiah_dbl = sunset( thedate, locinfo.lat, locinfo.longi, 90 + .83 );
   result.shkiah = convert_to_tz(thedate,result.shkiah_dbl,locinfo.tzname);
   try{
      result.tzait_hachochavim_dbl = sunset( thedate, locinfo.lat, locinfo.longi, 90 + 8.5 );
      result.tzait_hachochavim = convert_to_tz(thedate,result.tzait_hachochavim_dbl,locinfo.tzname);
   }catch(doesnthappen e){
      result.tzait_hachochavim = thedate;
      result.tzait_hachochavim.hour(-1);
      result.tzait_hachochavim.min(-1);
   }

   result.ph = ( result.shkiah_dbl - result.hanetz_dbl ) / 12; //proportional hour

   result.zmanshema     =  convert_to_tz(thedate,result.ph*3 + result.hanetz_dbl,locinfo.tzname);
   result.zmantefila    =  convert_to_tz(thedate,result.ph*4 + result.hanetz_dbl,locinfo.tzname);
   result.chatzot       =  convert_to_tz(thedate,result.ph*6 + result.hanetz_dbl,locinfo.tzname);
   result.minchagedola  =  convert_to_tz(thedate,result.ph*6.5 + result.hanetz_dbl,locinfo.tzname);
   result.minchakatanah =  convert_to_tz(thedate,result.ph*9.5 + result.hanetz_dbl,locinfo.tzname);
   result.plaghamincha  =  convert_to_tz(thedate,result.ph*10.75 + result.hanetz_dbl,locinfo.tzname);

   result.candles18 = convert_to_tz(thedate,result.shkiah_dbl-double(18)/double(60),locinfo.tzname);

   return result;
}

